#!/usr/bin/env python3

from collections import defaultdict
from glob import glob
from optparse import OptionParser
from os import path
from string import punctuation
from sys import argv
from xml.etree import ElementTree

import re


def errordie(msg):
    print(path.basename(argv[0]), ': error: ', msg, sep='')
    exit()

parser = OptionParser(usage='usage: %prog [options] NEWSROOTDIR')
parser.add_option('-d', '--hindi-normalization', action='store_true',
                  dest='hindi', default=False,
                  help="don't do Hindi normalization")
parser.add_option('-n', '--nospacemismatch', action='store_false',
                  dest='nospacemm', default=True,
                  help="don't remove NEWS data that has a word count mismatch")
parser.add_option('-p', '--handle-punctuation', action='store_false',
                  dest='handlepunc', default=True,
                  help="don't process the punctuation (for Hindi data only)")
parser.add_option('-s', '--splitwords', action='store_false',
                  dest='splitwords', default=True,
                  help="don't split NEWS strings into separate words")

(options, args) = parser.parse_args()
if not options.nospacemm:
    options.splitwords = False

if len(args) == 0:
    errordie('no XML file specified')
elif len(args) == 1:
    datafile = args[0]
elif len(args) > 1:
    errordie('too many arguments')

withAndWO = re.compile("['_^]")
smartReplace = [(re.compile('/'), ' '),
                (re.compile(',([^ ])'), r', \1'),
                (re.compile('[' + withAndWO.sub('', re.sub('-', '',
                                                           punctuation)) +
                            ']'), '')]
hindi = [(re.compile(chr(0x0958)), chr(0x0915) + chr(0x093C)),
         (re.compile(chr(0x0959)), chr(0x0916) + chr(0x093C)),
         (re.compile(chr(0x095A)), chr(0x0917) + chr(0x093C)),
         (re.compile(chr(0x095B)), chr(0x091C) + chr(0x093C)),
         (re.compile(chr(0x095C)), chr(0x0921) + chr(0x093C)),
         (re.compile(chr(0x095D)), chr(0x0922) + chr(0x093C)),
         (re.compile(chr(0x095E)), chr(0x092E) + chr(0x093C)),
         (re.compile(chr(0x095F)), chr(0x092F) + chr(0x093C))]
         #(re.compile(chr(0x200D)), '')] # not sure about this one

for directory in sorted(glob(path.join(args[0], 'En??/'))):
    lang = directory[-3:-1]
    handlepunc = lang == 'Hi' and options.handlepunc
    hindistuff = lang == 'Hi' and options.hindi
    transliterations = defaultdict(set)
    for datafile in sorted(glob(path.join(directory, '*.xml'))):
        for tlset in ElementTree.parse(datafile).getroot():
            src = tlset[0].text.strip().lower()
            if handlepunc:
                for cre, sub in smartReplace:
                    src = cre.sub(sub, src)
            srcs = src.split(' ')
            for entry in tlset[1:]:
                trg = entry.text.strip().lower()
                if handlepunc:
                    for cre, sub in smartReplace:
                        trg = cre.sub(sub, trg)
                if hindistuff:
                    for cre, sub in hindi:
                        trg = cre.sub(sub, trg)
                trgs = trg.split(' ')
                if not options.nospacemm or len(srcs) == len(trgs):
                    if options.splitwords:
                        for s, t in zip(srcs, trgs):
                            transliterations[s].add(t)
                            if handlepunc:
                                snew = withAndWO.sub('', s)
                                tnew = withAndWO.sub('', t)
                                transliterations[snew].add(tnew)
                    else:
                        transliterations[src].add(trg)
                        if handlepunc:
                            srcnew = withAndWO.sub('', src)
                            trgnew = withAndWO.sub('', trg)
                            transliterations[srcnew].add(trgnew)

    with open('En' + lang, 'w') as outp:
        print(lang, len(transliterations), sep='\t')
        for entry in sorted(transliterations):
            for transliteration in sorted(transliterations[entry]):
                outp.write('\t'.join([' '.join(entry),
                                      ' '.join(transliteration)]) + '\n')
